var JSLOADTIME = (new Date()).getTime();

var ConfigManager = TrustevV2 || (function () {
    return {
        BaseUrl: 'https://app.trustev.com/api/v2.0',
        PublicKey: '',
        EnableFP: false,
        BaseHost: 'app.trustev.com',
        CdnUrl: 'https://cdn.trustev.com',
        CookieName: 'tuid',
        ConsoleLoggingEnabled: false,
        Version: 'v3.9.30420'
    }
})();

var TrustevLogger = TrustevLogger || (function () {
    return {
        LogMessage: function (message) {
            if (ConfigManager.ConsoleLoggingEnabled) {
                var elapsedTime = (new Date().getTime()) - JSLOADTIME;
                console.log(elapsedTime + 'ms : ' + message);
            }
        }
    }
}());

var IsNotEmptyUUID = IsNotEmptyUUID || function isNotEmptyUUID(uuid) {
    return uuid !== null && uuid !== "null" && uuid !== "00000000-0000-0000-0000-000000000000" && uuid !== "" && uuid !== "undefined" && uuid !== undefined && uuid !== "NotSet";
}

var TrustevV2 = TrustevV2 || (function () {
    var _sessionId = '';
    var _deviceId = '';
    var _data = {};
    var _baseUrl = 'https://app.trustev.com/api/v2.0';
    var _ioBlackBox = [];
    var _shouldSendBlackbox = true;
    var _eventSubscribers = {};
    var configData = '';
    var _broadcast = function (eventname, args) {
        (_eventSubscribers[eventname] || function () {}).apply(undefined, args);
    };
    var _scriptUrlParams = '';
    var _isCookiesEnabled = false;
	
	
  function detectIsCameraEnabled(callback) {
        try {
            // Check if browser does not support MediaDevices API
            if (!navigator.mediaDevices || !navigator.mediaDevices.enumerateDevices) {
                callback(null);
                return;
            } else {
                navigator.mediaDevices.enumerateDevices()
                    .then(function(devices){
    
                        for (var i = 0; i < devices.length; i++)
                        {
                            if (devices[i].kind === 'videoinput') {
                                callback(true);
                                return;
                            }
                        }
                        callback(false);
                        return;
                    });
            }
        } catch (e) {
            TrustevLogger.LogMessage("TrustevV2.DoesDeviceSupportImageCapture: Error : " + e);
            callback(null);
            return;
        }
    };

    // inner function that returns the url params used on the script tag of this js call
    var _getScriptUrlParams = function () {

        function parseQuery(query) {
            var Params = new Object();
            if (!query) return Params; // return empty object
            var Pairs = query.split(/[;&]/);
            for (var i = 0; i < Pairs.length; i++) {
                var KeyVal = Pairs[i].split('=');
                if (!KeyVal || KeyVal.length != 2) continue;
                var key = unescape(KeyVal[0]);
                var val = unescape(KeyVal[1]);
                val = val.replace(/\+/g, ' ');
                Params[key] = val;
            }
            return Params;
        }

        // tests if script tag is the old trustevjs script tag
        function isTrustevScript(scriptSource) {
            if (scriptSource !== null && scriptSource.getAttribute('src') !== null) {
                return scriptSource.getAttribute('src').toLowerCase().indexOf("api/v2.0/trustevjs") > -1;
            }
            return false;
        }

        if (_scriptUrlParams === '') {
            var scripts = document.getElementsByTagName('script');
            var myScript = null;

            // cycle through all scripts to get the correct trustevJS script
            // have to this because of any possible async script calls, last script on the index wont be this script
            // if this is added as an async script
            for (var i = 0; i < scripts.length; ++i) {
                if (isTrustevScript(scripts[i])) {
                    myScript = scripts[i];
                    break;
                }
            }

            // getting all the query string params
            // myScript.src is "https://app.trustev.com/api/v2.0/trustevjs?key=key"
            if (myScript !== null) {
                var queryString = myScript.src.replace(/^[^\?]+\??/, '');
                _scriptUrlParams = parseQuery(queryString);
            }
        }
        return _scriptUrlParams;
    }

    var _getCookie = function (cname) {
        var name = cname + "=";
        var ca = document.cookie.split(';');
        for (var i = 0; i < ca.length; i++) {
            var c = ca[i];
            while (c.charAt(0) == ' ') {
                c = c.substring(1);
            }
            if (c.indexOf(name) == 0) {
                return c.substring(name.length, c.length);
            }
        }
        return "";
    }

    var _setCookie = function (key, val, expireInMinutes) {
        var expires = '';
        if (expireInMinutes > 0) {
            var d = new Date();
            d.setTime(d.getTime() + (expireInMinutes * 60 * 1000));
            var expires = "expires=" + d.toUTCString();
        }
        document.cookie = key + "=" + val + ";" + expires + "path=/; SameSite=None; Secure;";
    }

    var _checkCookiesEnabled = function () {
        var cookieEnabled = navigator.cookieEnabled;
        if (!cookieEnabled) {
            document.cookie = "testcookie";
            cookieEnabled = document.cookie.indexOf("testcookie") != -1;
        }
        return cookieEnabled;
    }

    return {
        BaseUrl: _baseUrl,
		 // check if the device supports a camera 
        DoesDeviceSupportImageCapture: detectIsCameraEnabled,
        Init: function (publicKey, callback, enableFP, baseUrl, cdnUrl) {
            ConfigManager.PublicKey = publicKey;
            ConfigManager.EnableFP = enableFP ? true : false;

            if (baseUrl) {
                _baseUrl = baseUrl;
                ConfigManager.BaseUrl = baseUrl;
            }

            if(cdnUrl)
            {
                ConfigManager.CdnUrl = cdnUrl;
            }

            //Lets go ahead and gather cookies and send them back to API
            function IframeInit() {
                if (document.body !== null) {
                    EvercookieManager.Init([QuickStorageManager, IFrameStorageManager], TrustevApiClient);
                    EvercookieManager.GatherEvercookiesAndSendToApi(ConfigManager.CookieName, function (evercookies) {
                        TrustevLogger.LogMessage('Finished getting evercookies : ' + JSON.stringify(evercookies));
                        EvercookieManager.SetMostAccurateEvercookie(ConfigManager.CookieName, evercookies);
                    });
                } else {
                    setTimeout(IframeInit, 1);
                }
            }

            if (_checkCookiesEnabled()) {
                IframeInit();
            }

            _sessionId = TrustevV2.GetCookie("tsid-" + ConfigManager.PublicKey);

            // Execute external services based on the config endpoint
            if (!_checkCookiesEnabled()) {
                TrustevV2.ExecuteExternalServices();
                TrustevLogger.LogMessage("Execute External Services If Part");
            } else {
                var jsConfigCookie = TrustevV2.GetCookie("jsConfig-" + ConfigManager.PublicKey);
                TrustevV2.ExecuteExternalServices(jsConfigCookie);
                TrustevLogger.LogMessage("Execute External Services Else Part");
            }

           
                if (_sessionId == "NotSet" || _sessionId == "") {

                    TrustevLogger.LogMessage('TrustevV2-Init : No sessionId found so calling /session endpoint');
                    // Generate session Id
                    TrustevApiClient.CallAPI("POST", _baseUrl + "/session", "{}", function () {
                        try {
                            _sessionId = JSON.parse(this.responseText).SessionId;
                            _deviceId = _sessionId;

                            if (configData.configuration.javascript.deviceId != null && IsNotEmptyUUID(_sessionId) &&
                                configData.configuration.javascript.deviceId.enabled === true && TrustevV2.IOBlackBox.length > 0) {
                                var blackBox = TrustevV2.IOBlackBox[TrustevV2.IOBlackBox.length - 1]
                                var bbRequest = {};
                                bbRequest.sessionId = _sessionId;
                                bbRequest.IsComplete = blackBox.complete;
                                bbRequest.blackBox = blackBox.blackBox;

                                TrustevApiClient.CallAPI("PUT", _baseUrl + "/ExtendedDevice", JSON.stringify(bbRequest), function () {
                                    TrustevLogger.LogMessage('TrustevV2-InitIovation : Successfully sent blackbox');
                                });
                            }

                            if (configData.configuration.javascript.passiveAuth != null &&
                                configData.configuration.javascript.passiveAuth.enabled === true) {
                                TrustevApiClient.CallAPI("GET", _baseUrl + "/configuration/" + _sessionId + "/GetPassiveAuthUrl", "{}", function () {
                                    var url = JSON.parse(this.responseText).PassiveAuthenticationURL;
                                    TrustevV2.PassiveAuthComplete(url);
                                });
                            }

                        } catch (e) {
                            TrustevLogger.LogMessage("TrustevV2-Init : Error on calling the session endpoint: " + e);
                        }

                        if (_sessionId != 'NotSet' && _sessionId != '') {
                            TrustevV2.SetCookie("tsid-" + ConfigManager.PublicKey, _sessionId, 20);
                        }

                        EvercookieManager.SetSessionId(_sessionId);

                        TrustevV2.SessionId = _sessionId;
                        TrustevV2.AsyncGatherInformation();

                        callback.apply(this, [_sessionId]);
                    });
                } else {
                    TrustevLogger.LogMessage('TrustevV2-Init : Existing sessionId found : ' + _sessionId);
                    TrustevV2.SessionId = _sessionId;
                    TrustevV2.AsyncGatherInformation();

                    EvercookieManager.SetSessionId(_sessionId);
                    callback.apply(this, [_sessionId]);
                }
        },
        ExecuteExternalServices: function (javascriptConfig) {
            // if no javascript config was provided then we get the configuration endpoint
            if (!javascriptConfig) {
                TrustevApiClient.CallAPI("GET", _baseUrl + "/configuration/javascript", "{}", function () {
                    try {
                        var configResponse = JSON.parse(this.responseText);
                        TrustevLogger.LogMessage("TrustevV2.ExecuteExternalServices: Configuration received successfully");
                        if (_checkCookiesEnabled()) {
                            TrustevV2.SetCookie("jsConfig-" + ConfigManager.PublicKey, JSON.stringify(configResponse), 5);
                        }
                        TrustevV2.ExecuteExternalServices(JSON.stringify(configResponse));
                    } catch (e) {
                        TrustevLogger.LogMessage("TrustevV2.ExecuteExternalServices: Error on calling the configuration/javascript endpoint: " + e);
                    }
                });
            } else {
                configData = JSON.parse(javascriptConfig);
                if (configData.configuration.javascript.deviceId != null &&
                    configData.configuration.javascript.deviceId.enabled === true) {

                    TrustevV2.InitIovation();
                }

                // Location enabled external services
                if (configData.configuration.javascript.locationServices != null &&
                    configData.configuration.javascript.locationServices.enabled === true) {
                    try {
                        TrustevV2.AsyncGatherLocationInfo(configData.configuration.javascript.locationServices.evURL);
                    } catch (e) {
                        TrustevLogger.LogMessage("TrustevV2.ExecuteExternalServices: Error on TrustevV2.AsyncGatherLocationInfo: " + e);
                    }
                }

                // Passive Auth enabled external services
                if (configData.configuration.javascript.passiveAuth != null &&
                    configData.configuration.javascript.passiveAuth.enabled === true && _sessionId !== "") {
                    TrustevApiClient.CallAPI("GET", _baseUrl + "/configuration/" + _sessionId + "/GetPassiveAuthUrl", "{}", function () {
                        var url = JSON.parse(this.responseText).PassiveAuthenticationURL;
                        TrustevV2.PassiveAuthComplete(url);
                    });
                }
            }
        },
        IOBlackBox: _ioBlackBox,
        InitIovation: function () {

            // Lets set time to stop sending black box
            setTimeout(function () {
                TrustevLogger.LogMessage('TrustevV2-InitIovation : Timing out black box sending after 5 seconds');
                _shouldSendBlackbox = false;
            }, 5000);

            window.io_global_object_name = "IGLOO";
            window.IGLOO = window.IGLOO || {
                "enable_flash": false,
                "bb_callback": function (bb, complete) {
                    var bbResponse = {
                        "blackBox": bb,
                        "complete": complete
                    };
                    TrustevV2.IOBlackBox.push(bbResponse);

                    var bbRequest = {};
                    bbRequest.sessionId = _sessionId;
                    bbRequest.IsComplete = complete;
                    bbRequest.blackBox = bb;
                    // send the black box data for up to 2 seconds to the endpoint
                    if (_shouldSendBlackbox && _sessionId !== "" && IsNotEmptyUUID(_sessionId)) {
                        TrustevApiClient.CallAPI("PUT", _baseUrl + "/ExtendedDevice", JSON.stringify(bbRequest), function () {
                            TrustevLogger.LogMessage('TrustevV2-InitIovation : Successfully sent blackbox');
                        });

                        if (complete) {
                            _shouldSendBlackbox = false;
                        }
                    } else {
                        TrustevLogger.LogMessage('TrustevV2-InitIovation : Not sending black box');
                    }
                },
                "loader": {
                    "fp_static": ConfigManager.EnableFP,
                    "subkey": "nLdMddw0jvDUZG3ETn8y0CEt7ZECk8Jm4N-uQ7wPu0k",
                    "version": "early5"
                }
            };

            /*
            IOvation script
            */
            (function A() {
                function u(a, b) {
                    var d = {},
                        c;
                    for (c = a.length - 1; - 1 < c; c--) 0 < c ? d[c] = function () {
                        var g = c;
                        return function () {
                            return v(a[g], d[g + 1], b)
                        }
                    }() : v(a[c], d[c + 1], b)
                }

                function v(f, m, d) {
                    var c = document.createElement("script"),
                        g = f[0].split("?")[0].split("/"),
                        e;
                    e = g[g.length - 1].split(".")[0];
                    g = a[d] && a[d].staticVer && a[d].staticVer + "/" || f[1];
                    /([0-9]+\.){2}[0-9]+\//.test(f[1]) && g !== f[1] && h("loader: Overriding configured version with staticVer.");
                    f[0] = f[0].replace("##version##", g);
                    c.setAttribute("src", f[0]);
                    c && c.addEventListener ?
                        c.addEventListener("error", function () {
                            b[d + "_" + e + "_load_failure"] = "true"
                        }) : c.attachEvent && c.attachEvent("onerror", function () {
                            b[d + "_" + e + "_load_failure"] = "true"
                        });
                    m && (c.onload = m);
                    document.getElementsByTagName("head")[0].appendChild(c)
                }

                function h(b) {
                    if ("function" === typeof a.trace_handler) try {
                        a.trace_handler(b)
                    } catch (e) {}
                }

                function e(a, b) {
                    var d = null !== a && void 0 !== a;
                    return !d || "1" !== a.toString() && "true" !== a.toString().toLowerCase() ? !d || "0" !== a.toString() && "false" !== a.toString().toLowerCase() ? "boolean" ===
                        typeof b ? b : !1 : !1 : !0
                }
                var t = window,
                    w = t.io_global_object_name || "IGLOO",
                    a = t[w] = t[w] || {},
                    b = a.loader = a.loader || {},
                    x = [],
                    y = [];
                if (b.loaderMain) return h("loader: Loader script has already run, try reducing the number of places it's being included."), !1;
                b.loaderMain = A;
                b.loaderVer = "5.1.0";
                (function () {
                    var f = e(b.tp, !0),
                        m = e(b.fp_static, !0),
                        d = e(b.fp_dyn, !0),
                        c = e(b.enable_legacy_compatibility),
                        g = e(b.tp_split),
                        u = b.tp_host && b.tp_host.replace(/\/+$/, "") || "https://mpsnare.iesnare.com",
                        z = b.fp_static_override_uri,
                        l = void 0 !==
                        b.uri_hook ? b.uri_hook + "/" : "/iojs/",
                        n = (b.version || "versionOrAliasIsRequired") + "/",
                        v = b.subkey ? t.encodeURIComponent(b.subkey) + "/" : "",
                        w = b.tp_resource || "wdp.js",
                        p = b.tp_host ? "&tp_host=" + t.encodeURIComponent(b.tp_host) : "",
                        B = z ? "&fp_static_uri=" + t.encodeURIComponent(z) : "",
                        q, r, k;
                    b.tp_host = u;
                    q = e(a.enable_flash, !0);
                    r = a.io && a.io.enable_flash;
                    k = a.fp && a.fp.enable_flash;
                    r = void 0 !== r && null !== r ? e(r, !0) : q;
                    void 0 !== k && null !== k ? k = e(k, !0) : r = q;
                    q = r ? "&flash=true" : "&flash=false";
                    k = k ? "&flash=true" : "&flash=false";
                    p = "?loaderVer=" +
                        b.loaderVer + "&compat=" + c + "&tp=" + f + "&tp_split=" + g + p + "&fp_static=" + m + "&fp_dyn=" + d + B;
                    f || m || h("loader: Not currently configured to load fp_static or tp script(s).");
                    a.fp && a.fp.staticVer && a.fp.staticVer + "/" !== n && (n = a.fp.staticVer + "/", h("loader: Configured version replaced with that from pre-loaded static script."));
                    m || a.fp && a.fp.staticMain ? (l = (l + "##version##" + v).replace(/\/\//g, "/"), m && (a.fp && a.fp.staticMain ? c && !a.fp.preCompatMain && h("loader: enable_legacy_compatibility on, but included static does not have the compat wrapper.") :
                        z ? x.push([z, ""]) : x.push([l + "static_wdp.js" + p + k, n])), !d || a.fp && a.fp.dynMain ? a.fp && a.fp.dynMain && h("loader: First party dynamic script has already been loaded, disable fp_dyn or make sure you're not manually including the dynamic file separately.") : x.push([l + "dyn_wdp.js" + p + k, n])) : e(b.fp_dyn) && h("loader: Invalid Config, first party dynamic script set to load without static.");
                    f && (a.io && a.io.staticMain ? h("loader: Third party script has already been loaded.") : (l = u + "/##version##" + v, g ? (y.push([l + "static_wdp.js" +
                        p + q, n
                    ]), y.push([l + "dyn_wdp.js" + p + q, n]), b.tp_resource && h("loader: Invalid Config: both tp_resource and tp_split set. Ignoring tp_resource.")) : y.push([l + w + p + q, n])))
                })();
                u(x, "fp");
                u(y, "io")
            })();

            /*
            END OF IOvation script
            */
        },
        SetCookie: _setCookie,
        GetCookie: _getCookie,
        AsyncGatherInformation: function () {
            if (_sessionId != "NotSet" && _sessionId != "") {
                setTimeout(function () {
                    TrustevV2.GatherInformation();
                }, 0);
            }
            return;
        },
        AsyncGatherLocationInfo: function (evurl) {
            setTimeout(function () {
                TrustevV2.LocationServices.CallEvUrl(evurl);
                TrustevV2.LocationServices.GetHtml5Location();
            }, 0);
            return;
        },
        GatherInformation: function (callback) {
            if (IsNotEmptyUUID(_sessionId)) {
                TrustevApiClient.CallAPI("POST", _baseUrl + "/Session/" + _sessionId + "/detail", "{}", function () { });
            }
            _data['SessionId'] = _sessionId;
            _data['DeviceId'] = _sessionId;

            try {
                _data['c5d1c702-f8bb-e311-be87-089e010f73f9'] = navigator.buildID != undefined ? navigator.buildID : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JavascriptData.NavigatorBuildID error: " + e);
            };
            try {
                _data['c6d1c702-f8bb-e311-be87-089e010f73f9'] = navigator.platform != undefined ? navigator.platform : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JavascriptData.NavigatorPlatform error: " + e);
            };
            try {
                _data['c7d1c702-f8bb-e311-be87-089e010f73f9'] = navigator.appCodeName != undefined ? navigator.appCodeName : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JavascriptData.NavigatorAppCodeName error: " + e);
            };
            try {
                _data['c8d1c702-f8bb-e311-be87-089e010f73f9'] = navigator.appVersion != undefined ? navigator.appVersion : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JavascriptData.NavigatorAppVersion error: " + e);
            };
            try {
                _data['c9d1c702-f8bb-e311-be87-089e010f73f9'] = navigator.appMinorVersion != undefined ? navigator.appMinorVersion : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JavascriptData.NavigatorAppMinorVersion error: " + e);
            };
            try {
                _data['cbd1c702-f8bb-e311-be87-089e010f73f9'] = navigator.cpuClass != undefined ? navigator.cpuClass : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JavascriptData.NavigatorCpuClass error: " + e);
            };
            try {
                _data['ccd1c702-f8bb-e311-be87-089e010f73f9'] = navigator.oscpu != undefined ? navigator.oscpu : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JavascriptData.NavigatorOscpu error: " + e);
            };
            try {
                _data['cdd1c702-f8bb-e311-be87-089e010f73f9'] = navigator.product != undefined ? navigator.product : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JavascriptData.NavigatorProduct error: " + e);
            };
            try {
                _data['ced1c702-f8bb-e311-be87-089e010f73f9'] = navigator.productSub != undefined ? navigator.productSub : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JavascriptData.NavigatorProductSub error: " + e);
            };
            try {
                _data['cfd1c702-f8bb-e311-be87-089e010f73f9'] = navigator.vendor != undefined ? navigator.vendor : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JavascriptData.NavigatorVendor error: " + e);
            };
            try {
                _data['d0d1c702-f8bb-e311-be87-089e010f73f9'] = navigator.vendorSub != undefined ? navigator.vendorSub : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JavascriptData.NavigatorVendorSub error: " + e);
            };
            try {
                _data['d1d1c702-f8bb-e311-be87-089e010f73f9'] = navigator.language != undefined ? navigator.language : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JavascriptData.NavigatorLanguage error: " + e);
            };
            try {
                _data['d2d1c702-f8bb-e311-be87-089e010f73f9'] = navigator.userLanguage != undefined ? navigator.userLanguage : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JavascriptData.NavigatorUserLanguage error: " + e);
            };
            try {
                _data['d3d1c702-f8bb-e311-be87-089e010f73f9'] = navigator.browserLanguage != undefined ? navigator.browserLanguage : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JavascriptData.NavigatorBrowserLanguage error: " + e);
            };
            try {
                _data['d4d1c702-f8bb-e311-be87-089e010f73f9'] = navigator.systemLanguage != undefined ? navigator.systemLanguage : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JavascriptData.NavigatorSystemLanguage error: " + e);
            };
            try {
                _data['d5d1c702-f8bb-e311-be87-089e010f73f9'] = navigator.doNotTrack != undefined ? navigator.doNotTrack : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JavascriptData.NavigatorDoNotTrack error: " + e);
            };
            try {
                _data['d6d1c702-f8bb-e311-be87-089e010f73f9'] = navigator.msDoNotTrack != undefined ? navigator.msDoNotTrack : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JavascriptData.NavigatorMsDoNotTrack error: " + e);
            };
            try {
                _data['d7d1c702-f8bb-e311-be87-089e010f73f9'] = navigator.javaEnabled() != undefined ? navigator.javaEnabled() : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JavascriptData.NavigatorJavaEnabled error: " + e);
            };
            try {
                _data['d8d1c702-f8bb-e311-be87-089e010f73f9'] = navigator.cookieEnabled != undefined ? navigator.cookieEnabled : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JavascriptData.NavigatorCookieEnabled error: " + e);
            };
            try {
                _data['d8406055-0ab1-e311-beb3-089e010f7621'] = navigator.userAgent != undefined ? navigator.userAgent : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JavascriptData.NavigatorUserAgent error: " + e);
            };
            try {
                _data['b886f45f-0ab1-e311-beb3-089e010f7621'] = navigator.appName != undefined ? navigator.appName : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JavascriptData.NavigatorAppName error: " + e);
            };
            try {
                var pluginData = [];
                var plugins = navigator.plugins;
                for (var i = 0; plugins[i]; ++i) {
                    var item = {};
                    item.name = plugins[i].name;
                    item.description = plugins[i].description;
                    item.filename = plugins[i].filename;
                    item.length = plugins[i].length;

                    pluginData.push(item);
                }
                _data['c5f0baba-92b6-e311-beb6-089e010f7621'] = JSON.stringify(pluginData);
            } catch (e) {
                TrustevLogger.LogMessage("Error in JavascriptData.NavigatorPlugins error: " + e);
            }
            try {
                _data['0e96e122-00bc-e311-be87-089e010f73f9'] = screen.width != undefined ? screen.width : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.ScreenWidth error: " + e);
            };
            try {
                _data['0f96e122-00bc-e311-be87-089e010f73f9'] = screen.height != undefined ? screen.height : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.ScreenHeight error: " + e);
            };
            try {
                _data['1096e122-00bc-e311-be87-089e010f73f9'] = screen.availWidth != undefined ? screen.availWidth : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.ScreenAvailWidth error: " + e);
            };
            try {
                _data['1196e122-00bc-e311-be87-089e010f73f9'] = screen.availHeight != undefined ? screen.availHeight : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.ScreenAvailHeight error: " + e);
            };
            try {
                _data['1296e122-00bc-e311-be87-089e010f73f9'] = screen.colorDepth != undefined ? screen.colorDepth : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.ScreenColorDepth error: " + e);
            };
            try {
                _data['1396e122-00bc-e311-be87-089e010f73f9'] = screen.pixelDepth != undefined ? screen.pixelDepth : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.ScreenPixelDepth error: " + e);
            };
            try {
                _data['1496e122-00bc-e311-be87-089e010f73f9'] = screen.fontSmoothingEnabled != undefined ? screen.fontSmoothingEnabled : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.ScreenFontSmoothingEnabled error: " + e);
            };
            try {
                _data['1596e122-00bc-e311-be87-089e010f73f9'] = screen.bufferDepth != undefined ? screen.bufferDepth : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.ScreenBufferDepth error: " + e);
            };
            try {
                _data['1696e122-00bc-e311-be87-089e010f73f9'] = screen.deviceXDPI != undefined ? screen.deviceXDPI : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.ScreenDeviceXDPI error: " + e);
            };
            try {
                _data['1796e122-00bc-e311-be87-089e010f73f9'] = screen.deviceYDPI != undefined ? screen.deviceYDPI : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.ScreenDeviceYDPI error: " + e);
            };
            try {
                _data['1996e122-00bc-e311-be87-089e010f73f9'] = screen.logicalXDPI != undefined ? screen.logicalXDPI : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.ScreenLogicalXDPI error: " + e);
            };
            try {
                _data['1a96e122-00bc-e311-be87-089e010f73f9'] = screen.logicalYDPI != undefined ? screen.logicalYDPI : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.ScreenLogicalYDPI error: " + e);
            };
            try {
                _data['1b96e122-00bc-e311-be87-089e010f73f9'] = screen.systemXDPI != undefined ? screen.systemXDPI : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.ScreenSystemXDPI error: " + e);
            };
            try {
                _data['1c96e122-00bc-e311-be87-089e010f73f9'] = screen.systemYDPI != undefined ? screen.systemYDPI : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.ScreenSystemYDPI error: " + e);
            };
            try {
                _data['1d96e122-00bc-e311-be87-089e010f73f9'] = screen.updateInterval != undefined ? screen.updateInterval : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.ScreenUpdateInterval error: " + e);
            };
            try {
                _data['1e96e122-00bc-e311-be87-089e010f73f9'] = window.devicePixelRatio != undefined ? window.devicePixelRatio : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.WindowDevicePixelRatio error: " + e);
            };
            try {
                _data['1f96e122-00bc-e311-be87-089e010f73f9'] = window.outerWidth != undefined ? window.outerWidth : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.WindowOuterWidth error: " + e);
            };
            try {
                _data['2096e122-00bc-e311-be87-089e010f73f9'] = window.outerHeight != undefined ? window.outerHeight : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.WindowOuterHeight error: " + e);
            };
            try {
                _data['2196e122-00bc-e311-be87-089e010f73f9'] = window.innerWidth != undefined ? window.innerWidth : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.WindowInnerWidth error: " + e);
            };
            try {
                _data['2296e122-00bc-e311-be87-089e010f73f9'] = window.innerHeight != undefined ? window.innerHeight : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.WindowInnerHeight error: " + e);
            };
            try {
                _data['2396e122-00bc-e311-be87-089e010f73f9'] = window.screenX != undefined ? window.screenX : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.WindowScreenX error: " + e);
            };
            try {
                _data['2496e122-00bc-e311-be87-089e010f73f9'] = window.screenY != undefined ? window.screenY : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.WindowScreenY error: " + e);
            };
            try {
                _data['2596e122-00bc-e311-be87-089e010f73f9'] = window.screenTop != undefined ? window.screenTop : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.WindowScreenTop error: " + e);
            };
            try {
                _data['2696e122-00bc-e311-be87-089e010f73f9'] = window.screenLeft != undefined ? window.screenLeft : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.WindowScreenLeft error: " + e);
            };
            try {
                _data['2796e122-00bc-e311-be87-089e010f73f9'] = document.documentElement.clientWidth != undefined ? document.documentElement.clientWidth : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.DocumentDocumentElementClientWidth error: " + e);
            };
            try {
                _data['2896e122-00bc-e311-be87-089e010f73f9'] = document.documentElement.clientHeight != undefined ? document.documentElement.clientHeight : "undefined";
            } catch (e) {
                TrustevLogger.LogMessage("Error in JSDisplayData.DocumentDocumentElementClientHeight error: " + e);
            };

            if (IsNotEmptyUUID(_sessionId)) {
                TrustevApiClient.CallAPI("POST", _baseUrl + "/device", JSON.stringify(_data), function (xhr, json) {
                    if (callback !== undefined) {
                        return callback(xhr, json);
                    }
                    TrustevV2.SetCookie("tsid-" + ConfigManager.PublicKey, _sessionId, 20);

                    _broadcast('deviceInfoCollected', [_sessionId]);
                });
            }

            return;
        },
        SessionId: (function () {

            if (_sessionId !== 'NotSet' && _sessionId !== '') {
                return _sessionId;
            } else {
                if (ConfigManager.PublicKey === '' || ConfigManager.PublicKey === 'NotSet') {
                    // getting the publickKey if available on the script tag
                    // current script executing is always the last one of the scripts list
                    var params = _getScriptUrlParams();

                    if (!params.hasOwnProperty('key')) {
                        return;
                    }

                    ConfigManager.PublicKey = params["key"];
                }

                _sessionId = _getCookie("tsid-" + ConfigManager.PublicKey);

                if (_sessionId === 'NotSet' || _sessionId === '') {
                    TrustevLogger.LogMessage('TrustevV2-Init : No sessionId found so calling /session endpoint');

                    try {
                        var xhr = null;

                        if (window.XMLHttpRequest) {
                            xhr = new XMLHttpRequest();
                        } else if (window.ActiveXObject) {
                            xhr = new ActiveXObject("Microsoft.XMLHTTP");
                        }

                        if (xhr) {

                            xhr.open("POST", _baseUrl + "/session", false);
                            xhr.setRequestHeader("X-PublicKey", ConfigManager.PublicKey);
                            xhr.setRequestHeader("Content-Type", "application/json");
                            xhr.setRequestHeader("X-TU-DV-JS-Version", ConfigManager.Version);

                            xhr.send("{}");
                            if (xhr.status === 200) {
                                _sessionId = JSON.parse(xhr.responseText).SessionId;
                                _deviceId = _sessionId;

                                _setCookie("tsid-" + ConfigManager.PublicKey, _sessionId, 20)
                            }
                        }
                    } catch (e) {
                        TrustevLogger.LogMessage('TrustevV2-Init : Error on calling the /session endpoint');;
                    }
                }

                return _sessionId;
            }
        }()),
        DeviceId: (_data['DeviceId'] || _deviceId),
        LocationServices: {
            UpdateSessionPosition: function (position) {
                var data = {
                    SessionId: TrustevV2.SessionId,
                    Latitude: position.coords.latitude,
                    Longitude: position.coords.longitude
                }

                // updating the session with location information
                TrustevApiClient.CallAPI("POST", _baseUrl + "/location", JSON.stringify(data), function () {});
            },

            GetHtml5Location: function () {
                if (navigator.geolocation) {
                    navigator.geolocation.getCurrentPosition(this.UpdateSessionPosition);
                } else {
                    TrustevLogger.LogMessage("Geolocation is not supported by this browser.");
                }
            },
            CallEvUrl: function (evurl) {
                // create an img tag to get the evUrl.
                // This "hack" allows us to bypass the mixed content error
                var img = document.createElement("img");
                img.src = evurl;
                var head = document.getElementsByTagName('head')[0];
                head.appendChild(img);
            }
        },
        PassiveAuthComplete: function (authUrl) {
            var img = document.createElement("img");
            img.src = authUrl;
            var head = document.getElementsByTagName('head')[0];
            head.appendChild(img);
        },
        On: function (eventName, callback) {
            _eventSubscribers[eventName] = callback;
        },
        GetJsUrlParams: _getScriptUrlParams()
    };
}());

var TrustevApiClient = TrustevApiClient || (function () {
    return {
        CallAPI: function (method, url, json, callback, headers) {
            try {
                var xhr = null;

                if (window.XMLHttpRequest) {
                    xhr = new XMLHttpRequest();
                } else if (window.ActiveXObject) {
                    xhr = new ActiveXObject("Microsoft.XMLHTTP");
                }

                if (xhr) {
                    xhr.onreadystatechange = function () {
                        if (xhr.readyState == XMLHttpRequest.DONE) {
                            callback.apply(this, [xhr, json]);
                        }
                    }

                    xhr.open(method, url, true);

                    // setting headers if they were specified
                    if (headers !== null && headers !== undefined) {
                        for (var key in headers) {
                            if (headers.hasOwnProperty(key)) {
                                xhr.setRequestHeader(key, headers[key]);
                            }
                        }
                    } else {
                        xhr.setRequestHeader("X-PublicKey", ConfigManager.PublicKey);
                        xhr.setRequestHeader("Content-Type", "application/json");
                        xhr.setRequestHeader("X-TU-DV-JS-Version", ConfigManager.Version);
                    }

                    if (method === "GET") {
                        xhr.send();
                    } else {
                        xhr.send(json);
                    }


                }
            } catch (e) {
                TrustevLogger.LogMessage("Error on CallAPI - TrustevApiClient: " + e);
            }
        },
    }
})();

var EvercookieManager = EvercookieManager || (function () {
    _selectedStorageManagers = [];
    _trustevApiClient = null
    _sessionId = null;
    _isInitiated = false;
    _queuedEvercookies = [];
    _completeManagerCount = 0;
    _gatherEvercookieCallback = null;
    _gatheredEvercookies = []

    return {
        Init: function (selectedStorageManagers, trustevApiClient) {
            _selectedStorageManagers = selectedStorageManagers;
            _trustevApiClient = trustevApiClient;
            _isInitiated = true;
        },
        SetSessionId: function (sessionId) {
            _sessionId = sessionId;
            TrustevLogger.LogMessage('EvercookieManager-SetSessionId : Setting SessionId : ' + sessionId);

            //Here we need to send over any cookies that have been waiting for a sessionId
            if (_queuedEvercookies.length > 0) {
                for (var i = 0; i < _queuedEvercookies.length; i++) {
                    this.SendEvercookieToApi(_queuedEvercookies[i], this.TrustevApiCallback)
                }
            }

            if (_completeManagerCount > 0 &&
                _completeManagerCount === _selectedStorageManagers.length) {
                var initEvercookie = {};
                initEvercookie.source = 'init';
                initEvercookie.id = _sessionId;

                this.SendEvercookieToApi(initEvercookie, this.TrustevApiCallback);
            }
        },
        GatherEvercookiesAndSendToApi: function (evercookieKey, callback) {
            _gatherEvercookieCallback = callback;
            var self = this;

            if (!_isInitiated)
                throw 'You need to Init DeviceTagManager before executing'

            for (var i = 0; i < _selectedStorageManagers.length; i++) {
                _selectedStorageManagers[i].Get(evercookieKey, function (evercookie) {
                    if (evercookie.id) {
                        TrustevLogger.LogMessage('EvercookieManager-GatherEvercookiesAndSendToApi : Storage manager returned cookie : ' + JSON.stringify(evercookie));
                        _gatheredEvercookies.push(evercookie);

                        //If we have a sessionId lets send the cookie accross
                        if (_sessionId) {
                            self.SendEvercookieToApi(evercookie, self.TrustevApiCallback);
                        } else {
                            //Otherwise lets store if until we have a sessionId
                            _queuedEvercookies.push(evercookie);
                        }
                    } else {
                        _completeManagerCount += 1;

                        if (_completeManagerCount === _selectedStorageManagers.length && _sessionId) {
                            //We have not found any evercookies so going to send sessionId as id and init as source
                            var initEvercookie = {};
                            initEvercookie.source = 'init';
                            initEvercookie.id = _sessionId;

                            self.SendEvercookieToApi(initEvercookie, self.TrustevApiCallback);
                        }
                    }
                });
            }
        },
        SetMostAccurateEvercookie: function (evercookieKey, evercookies, callback) {
            var didWeFindAnEvercookie = false;
            var didWeFindAnIFrameEvercookie = false;

            if (evercookies && evercookies.length > 0) {
                for (var i = 0; i < evercookies.length; i++) {
                    if (evercookies[i].id) {
                        didWeFindAnEvercookie = true;

                        if (evercookies[i].source.indexOf('iframe-') !== 1) {
                            TrustevLogger.LogMessage('EvercookieManager-SetMostAccurateEvercookie : Found an iframe evercookie so setting that. Evercookie : ' + JSON.stringify(evercookies[i]));
                            for (var j = 0; j < _selectedStorageManagers.length; j++)
                                _selectedStorageManagers[j].Set(evercookieKey, evercookies[i].id);

                            didWeFindAnIFrameEvercookie = true;
                            break;
                        }
                    }
                }
            }

            if (didWeFindAnEvercookie && !didWeFindAnIFrameEvercookie) {
                for (var k = 0; k < evercookies.length; k++) {
                    if (evercookies[k].id) {
                        TrustevLogger.LogMessage('EvercookieManager-SetMostAccurateEvercookie : Didnt find an iframe evercookie so using local domain one. Evercookie : ' + JSON.stringify(evercookies[k]));
                        for (var l = 0; l < _selectedStorageManagers.length; l++)
                            _selectedStorageManagers[l].Set(evercookieKey, evercookies[k].id);
                    }
                }
            }

            //If we have not found any valid evercookies. Lets set it to sessionId
            if (!didWeFindAnEvercookie)
                for (var m = 0; m < _selectedStorageManagers.length; m++) {
                    TrustevLogger.LogMessage('EvercookieManager-SetMostAccurateEvercookie : Didnt find any evercookies, so going to use sessionId : ' + JSON.stringify(_sessionId));
                    _selectedStorageManagers[m].Set(evercookieKey, _sessionId);
                }

            if (callback)
                callback();
        },
        TrustevApiCallback: function () {
            //Once the cookie has been successfully sent we check to see if have completed all the searches
            _completeManagerCount += 1;
            if (_completeManagerCount >= _selectedStorageManagers.length)
                if (_gatherEvercookieCallback)
                    _gatherEvercookieCallback(_gatheredEvercookies);
        },
        SendEvercookieToApi: function (evercookie, callback) {
            var data = {};
            data['EvercookieId'] = evercookie.id;
            data['EvercookieSource'] = evercookie.source;
            data['SessionId'] = _sessionId;
            data['DeviceId'] = _sessionId;
            data['PublicKey'] = ConfigManager.PublicKey;
            
            TrustevLogger.LogMessage('EvercookieManager-SendEvercookieToApi : Sending a cookie to API : ' + JSON.stringify(evercookie));

            _trustevApiClient.CallAPI("POST", ConfigManager.BaseUrl + "/device", JSON.stringify(data), callback);
        }
    }
}());

var SessionStorageManager = SessionStorageManager || (function () {
    var sessionStorage = null;
    try {
        sessionStorage = window.sessionStorage;
    } catch (e) {
        TrustevLogger.LogMessage("sessionStorage - error: " + e.message + "; " + e.description);
    }

    if (sessionStorage !== null) {
        return {
            Get: function (evercookieIdKey, callback) {
                var evercookie = {}
                evercookie.source = "sessionstorage";
                try {
                    if (sessionStorage) {
                        var originalValue = sessionStorage.getItem(evercookieIdKey);
                        if (originalValue !== undefined && originalValue !== null)
                            evercookie.id = originalValue;
                    }
                } catch (e) {
                    TrustevLogger.LogMessage("evercookie_session_storage - error: " + e.message + "; " + e.description);
                }
                callback(evercookie);
            },
            Set: function (evercookieIdKey, evercookieId, callback) {
                sessionStorage.setItem(evercookieIdKey, evercookieId);

                if (callback)
                    callback();
            }
        }
    }

    return null;

}());

var LocalStorageManager = LocalStorageManager || (function () {
    var localStorage = null;

    try {
        localStorage = window.localStorage;
    } catch (e) {
        TrustevLogger.LogMessage("localStorage - error: " + e.message + "; " + e.description);
    }

    if (localStorage !== null) {
        return {
            Get: function (evercookieIdKey, callback) {
                var evercookie = {};
                evercookie.source = "localstorage";

                try {
                    if (localStorage) {
                        var originalValue = localStorage.getItem(evercookieIdKey);
                        if (originalValue !== undefined && originalValue !== null)
                            evercookie.id = originalValue;
                    }
                } catch (e) {
                    TrustevLogger.LogMessage("evercookie_local_storage - error: " + e.message + "; " + e.description);
                }

                callback(evercookie);
            },
            Set: function (evercookieIdKey, evercookieId, callback) {
                localStorage.setItem(evercookieIdKey, evercookieId);

                if (callback)
                    callback();
            }
        }

        return null;
    }
}());

var GlobalStorageManager = GlobalStorageManager || (function () {
    globalStorage = window.globalStorage;
    host = window.location.host.replace(/:\d+/, '');
    return {
        Get: function (evercookieIdKey, callback) {
            var evercookie = {}
            evercookie.source = 'globalstorage';

            if (globalStorage) {
                globalStorageEvercookie.id = 'h1';
                var originalValue = globalStorage[host][evercookieIdKey];
                if (originalValue !== undefined && originalValue !== null)
                    evercookie.id = originalValue;
            }
            callback(evercookie);
        },
        Set: function (evercookieIdKey, evercookieId, callback) {
            if (globalStorage)
                globalStorage[host][evercookieIdKey] = evercookieId;

            if (callback)
                callback();
        }
    }
}());

var IndexDbStorageManager = IndexDbStorageManager || (function () {
    indexDbStorageEvercookie = {};
    indexDbStorageEvercookie.source = "indexdb";

    return {
        Get: function (evercookieIdKey, callback) {
            try {
                if (!('indexedDB' in window)) {
                    indexedDB = window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
                    IDBTransaction = window.IDBTransaction || window.webkitIDBTransaction || window.msIDBTransaction;
                    IDBKeyRange = window.IDBKeyRange || window.webkitIDBKeyRange || window.msIDBKeyRange;
                }

                if (indexedDB) {
                    var ver = 1;
                    //FF incognito mode restricts indexedb access
                    var request = indexedDB.open("idb_evercookie", ver);

                    request.onerror = function (e) {
                        TrustevLogger.LogMessage("evercookie_indexdb_storage - error: " + e);
                        callback(indexDbStorageEvercookie);
                    }

                    request.onupgradeneeded = function (event) {
                        var db = event.target.result;

                        var store = db.createObjectStore("evercookie", {
                            keyPath: "name",
                            unique: false
                        });
                    }
                    request.onsuccess = function (event) {
                        var idb = event.target.result;
                        if (idb.objectStoreNames.contains("evercookie")) {
                            var tx = idb.transaction(["evercookie"], "readwrite");
                            var objst = tx.objectStore("evercookie");
                            var qr = objst.get(evercookieIdKey);

                            qr.onsuccess = function (event) {
                                if (qr.result !== undefined) {
                                    var originalValue = qr.result.value;
                                    if (originalValue !== undefined && originalValue !== null) {
                                        indexDbStorageEvercookie.id = originalValue;
                                    }
                                }

                                callback(indexDbStorageEvercookie)
                            }
                        }
                        idb.close();
                    }
                } else {
                    callback(indexDbStorageEvercookie)
                }
            } catch (e) {
                TrustevLogger.LogMessage("evercookie_indexdb_storage - error: " + err);
                callback(indexDbStorageEvercookie);
            }

        },
        Set: function (evercookieIdKey, evercookieId, callback) {
            try {
                if (indexedDB) {
                    var ver = 1;
                    //FF incognito mode restricts indexedb access
                    var request = indexedDB.open("idb_evercookie", ver);

                    request.onerror = function (e) {
                        TrustevLogger.LogMessage("evercookie_indexdb_storage - error: " + e);
                        if (callback)
                            callback();
                    }

                    request.onupgradeneeded = function (event) {
                        var db = event.target.result;

                        var store = db.createObjectStore("evercookie", {
                            keyPath: "name",
                            unique: false
                        });
                        if (callback)
                            callback();
                    }

                    request.onsuccess = function (event) {
                        var idb = event.target.result;
                        if (idb.objectStoreNames.contains("evercookie")) {
                            var tx = idb.transaction(["evercookie"], "readwrite");
                            var objst = tx.objectStore("evercookie");
                            var qr = objst.put({
                                "name": evercookieIdKey,
                                "value": evercookieId
                            });
                        }
                        idb.close();
                        if (callback)
                            callback();
                    }
                } else {
                    if (callback)
                        callback();
                }
            } catch (e) {
                TrustevLogger.LogMessage("evercookie_indexdb_storage - error: " + err);
                if (callback)
                    callback();

            }
        }
    }

}());

var DatabaseStorageManager = DatabaseStorageManager || (function () {
    databaseStorageEvercookie = {};
    databaseStorageEvercookie.source = "databasestorage";
    database = null;

    return {
        Get: function (evercookieIdKey, callback) {
            if (window.openDatabase) {
                if (!database)
                    database = window.openDatabase("etrust", "", evercookieIdKey, 1024 * 1024);

                database.transaction(function (tx) {
                    tx.executeSql("SELECT value FROM cache WHERE name=?", [evercookieIdKey],
                        function (tx, result1) {
                            if (result1.rows.length >= 1) {
                                var originalValue = result1.rows.item(0).value;
                                if (originalValue !== undefined && originalValue !== null)
                                    databaseStorageEvercookie.id = originalValue;
                            }

                            callback(databaseStorageEvercookie);
                        },
                        function (tx, err) {
                            TrustevLogger.LogMessage("evercookie_database_storage - error: " + err);
                            callback(databaseStorageEvercookie);
                        });
                });
            } else {
                callback(databaseStorageEvercookie);
            }
        },
        Set: function (evercookieIdKey, evercookieId, callback) {
            if (window.openDatabase) {
                if (!database)
                    database = window.openDatabase("etrust", "", evercookieIdKey, 1024 * 1024);

                database.transaction(function (tx) {
                    tx.executeSql("CREATE TABLE IF NOT EXISTS cache(" +
                        "id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, " +
                        "name TEXT NOT NULL, " +
                        "value TEXT NOT NULL, " +
                        "UNIQUE (name)" +
                        ")", [],
                        function (tx, rs) {},
                        function (tx, err) {});
                    tx.executeSql("INSERT OR REPLACE INTO cache(name, value) " +
                        "VALUES(?, ?)",
                        [evercookieIdKey, evercookieId],
                        function (tx, rs) {},
                        function (tx, err) {});
                });
                if (callback)
                    callback();
            } else {
                if (callback)
                    callback();
            }
        }
    }

}());

var QuickStorageManager = QuickStorageManager || (function () {
    quickStorageManagers = [LocalStorageManager, SessionStorageManager, GlobalStorageManager];

    return {
        Get: function (evercookieIdKey, callback) {
            var callbackCount = 0;
            var isSetCallbackExecuted = false;

            for (var i = 0; i < quickStorageManagers.length; i++) {
                quickStorageManagers[i].Get(evercookieIdKey, function (evercookie) {
                    if (evercookie.id && !isSetCallbackExecuted) {
                        isSetCallbackExecuted = true;
                        callback(evercookie)
                    }

                    if (++callbackCount == quickStorageManagers.length) {
                        if (!isSetCallbackExecuted)
                            callback(evercookie)

                        isSetCallbackExecuted = false;
                    }

                });
            }
        },
        Set: function (evercookieIdKey, evercookieId, callback) {
            var callbackCount = 0;

            for (var i = 0; i < quickStorageManagers.length; i++) {
                quickStorageManagers[i].Set(evercookieIdKey, evercookieId, function (evercookie) {
                    if (++callbackCount === quickStorageManagers.length)
                        if (callback)
                            callback();
                });
            }
        }
    }
}());

var IFrameStorageManager = IFrameStorageManager || (function () {
    iframe = null;
    loadIFrame = function (callback) {
        iframe = document.createElement('iframe');
        iframe.setAttribute("src", ConfigManager.CdnUrl + '/trustevIFrame.html');
        iframe.style.display = 'none';
        document.body.appendChild(iframe);

        iframe.onload = function () {
            IFrameMessageManager.Init(window, 'message');
            callback();
        }

    };
    return {
        Get: function (evercookieIdKey, callback) {
            var evercookie = {};
            evercookie.source = "iframe";
            var message = {};
            message.type = 'IFrame_Evercookie_Get_Init'
            message.evercookieIdKey = evercookieIdKey;

            if (!iframe)
                loadIFrame(function () {
                    IFrameMessageManager.Send(iframe, message, function (evercookie) {
                        evercookie.source = 'iframe-' + evercookie.source;
                        TrustevLogger.LogMessage('IFrameStorageManager-Get : Found IFrame evercookie : ' + JSON.stringify(evercookie));
                        callback(evercookie);
                    });
                });
            else {
                IFrameMessageManager.Send(iframe, message, function (evercookie) {
                    evercookie.source = 'iframe-' + evercookie.source;
                    TrustevLogger.LogMessage('IFrameStorageManager-Get : Found IFrame evercookie : ' + JSON.stringify(evercookie));
                    callback(evercookie);
                });
            }


        },
        Set: function (evercookieIdKey, evercookieId, callback) {
            var message = {};
            message.type = 'IFrame_Evercookie_Set_Init'
            message.evercookieIdKey = evercookieIdKey;
            message.evercookieId = evercookieId;
            TrustevLogger.LogMessage('Setting IFrame evercookie : Key ' + evercookieIdKey + ', Id : ' + evercookieId);
            if (!iframe) {
                loadIFrame(function () {
                    IFrameMessageManager.Send(iframe, message, callback);
                });
            } else {
                IFrameMessageManager.Send(iframe, message, callback);
            }
        }
    }
}());

var IFrameMessageManager = IFrameMessageManager || (function () {
    messageIdentifier = 'TrustevIFrameMessage';
    currentMessageCallBack = null;
    iFrameListener = function (event) {
        if (event.data && event.data.identifier === 'TrustevIFrameMessage') {
            switch (event.data.type) {
                case 'IFrame_Evercookie_Get_Result':
                    if (currentMessageCallBack)
                        currentMessageCallBack(event.data.evercookie);
                    break;
                case 'IFrame_Evercookie_Set_Result':
                    if (currentMessageCallBack)
                        currentMessageCallBack();
                    break;
                default:
                    TrustevLogger.LogMessage('Unrecognized iframe message type');
                    break;
            }
        }
    };
    return {
        Init: function (obj, evt, func) {
            if ('addEventListener' in obj) {
                obj.addEventListener(evt, iFrameListener, false);
            } else if ('attachEvent' in obj) { //IE
                obj.attachEvent('on' + evt, iFrameListener);
            }
        },
        Send: function (iframe, message, callback) {
            currentMessageCallBack = callback;
            message.identifier = messageIdentifier;
            iframe.contentWindow.postMessage(message, "*");
        }
    }
}());

// self executing function that calls TrustevV2.Init method if publicKey was provided and TrustevV2 is available
(function InitializeApiJs() {

    if (typeof TrustevV2 !== "undefined") {

        var params = TrustevV2.GetJsUrlParams;

        if (params.hasOwnProperty('key')) {
            TrustevV2.Init(params["key"], function () {})
        }
    } else {
        setTimeout(InitializeApiJs, 50)
    }
}());